@@ -7,6 +7,7 @@
 #' @importFrom Matrix sparseMatrix
 #' @importFrom imager as.cimg erode_square isoblur boundary px.circle
 #' @importFrom dplyr group_by group_keys group_split
+#' @importFrom rlang as_label
 #' @return a `ggbrain_layer_outline` R6 class with fields related to a brain visual layer (relates to `geom_outline`)
 #' @export
 ggbrain_layer_outline <- R6::R6Class(
@@ -142,7 +143,24 @@ ggbrain_layer_outline <- R6::R6Class(
         if (is.null(value$outline)) {
           private$pvt_fill_column <- NULL
         } else {
-          private$pvt_fill_column <- rlang::as_name(value$outline) # pull out the outline column from aes
+          private$pvt_fill_column <- all.vars(value$outline) # pull out the outline column from aes
+          if (length(private$pvt_fill_column) > 1L) stop("Cannot pass multiple columns as a fill mapping")
+
+          # if the user modifies the fill column such as as.factor(), we need to carry through the expression to geom_raster
+          # inside the add_raster method of ggbrain_layer, we use 'new_val' as the column name to plot. Create a glue expression here
+          # that will be evaluated at the time of the add_raster.
+          fill_expr <- rlang::as_label(value$outline)
+          if (fill_expr != private$pvt_fill_column) {
+            private$pvt_fill_expr <- sub(private$pvt_fill_column, "{new_val}", fill_expr, fixed = TRUE)
+            # if user is wrapping column name in factor or as.factor, treat as factor internally
+            # also strip factor conversion from fill expression so that ggplot doesn't relevel the factor inline, undermining unify_scales
+            if (private$pvt_fill_expr == "factor({new_val})" ||
+              private$pvt_fill_expr == "as.factor({new_val})") {
+              private$pvt_categorical_fill <- TRUE
+              private$pvt_fill_expr <- sub("(as.)*factor\\({new_val}\\)", "{new_val}", private$pvt_fill_expr, perl = TRUE)
+            }
+          }
+
           # always pass through the group as the fill column so that the outline conversion gets the grouping right
           private$pvt_group_column <- private$pvt_fill_column
           private$pvt_has_fill <- TRUE
